# Copyright (c) 2015 The University of Illinois Urbana Champaign
# All rights reserved
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met: redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer;
# redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution;
# neither the name of the copyright holders nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Authors: Mohammad Alian

# This is an example of an n port network switch to work in dist-gem5.
# Users can extend this to have different different topologies

import optparse
import sys

import m5
from m5.defines import buildEnv
from m5.objects import *
from m5.util import addToPath, fatal

addToPath('../common')

import Simulation
import Options

def build_switch_star(options, system):
    # instantiate an EtherSwitch
    system.switch = EtherSwitch(fabric_speed = options.ethernet_switchspeed,
                                 delay = options.ethernet_switchdelay)
    # instantiate distEtherLinks to connect switch ports
    # to other gem5 instances
    system.portlink = [DistEtherLink(speed = options.ethernet_linkspeed,
                                      delay = options.ethernet_linkdelay,
                                      dist_rank = options.dist_rank,
                                      dist_size = options.dist_size,
                                      server_name = options.dist_server_name,
                                      server_port = options.dist_server_port,
                                      sync_start = options.dist_sync_start,
                                      sync_repeat = options.dist_sync_repeat,
                                      is_switch = True,
                                      num_nodes = options.dist_size)
                       for i in xrange(options.dist_size)]

    for (i, link) in enumerate(system.portlink):
        link.int0 = system.switch.interface[i]

def build_switch_tree2(options, system):

    # instantiate EtherSwitches
    system.switch = [EtherSwitch(fabric_speed = options.ethernet_switchspeed,
                                  delay = options.ethernet_switchdelay)
                        for i in xrange(3)]

    # instantiate distEtherLinks to connect top or rack switch ports
    # to other gem5 instances
    system.portlink = [DistEtherLink(speed = options.ethernet_linkspeed,
                                      delay = options.ethernet_linkdelay,
                                      dist_rank = options.dist_rank,
                                      dist_size = options.dist_size,
                                      server_name = options.dist_server_name,
                                      server_port = options.dist_server_port,
                                      sync_start = options.dist_sync_start,
                                      sync_repeat = options.dist_sync_repeat,
                                      is_switch = True,
                                      num_nodes = options.dist_size)
                       for i in xrange(options.dist_size)]

    # connect distEtherLinks to top of rack switches
    for (i, link) in enumerate(system.portlink):
        link.int0 = system.switch[int(i * 2 / options.dist_size)]\
                    .interface[i % int(options.dist_size / 2)]

    system.aggrlink = [EtherLink(speed = options.ethernet_linkspeed,
                                      delay = options.ethernet_linkdelay)
                       for i in xrange(2)]
    for (i, link) in enumerate(system.aggrlink):
        link.int0 = system.switch[2].interface[i]
        link.int1 = system.switch[i].interface[int(options.dist_size / 2)]

# Add options
parser = optparse.OptionParser()
Options.addCommonOptions(parser)
Options.addFSOptions(parser)
(options, args) = parser.parse_args()

system = Root(full_system = True)
if options.ethernet_topology == "tree":
    build_switch_tree2(options, system)
else:
    build_switch_star(options, system)

Simulation.run(options, system, None, None)

